##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import os, zLOG, random, string, sys
from Globals import package_home, InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_inner, aq_parent
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.Expression import Expression, createExprContext
from Products.PageTemplates import PageTemplateFile
from Products.MyMediaManager.Medianet import registerPlugin
from Products.MyMediaManager.interfaces.IEncoderPlugin import IEncoderPlugin
from Products.MyMediaManager.PluginBase import PluginBase

_zptdir = os.path.join(package_home(globals()), 'www')

class EncoderBase(SimpleItem):

    meta_type = 'Encoder Base'
    _profiles = []
    _expression = None

    security = ClassSecurityInfo()

    _profilestructure = [
                            {"id":"width","title":"Width","type":"int","value":180}
                           ,{"id":"height","title":"Height","type":"int","value":160} 
                        ]
    manage_options = ( { 'label' : 'Profile', 'action' : 'manage_profiles' },)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_profiles')
    manage_profiles = PageTemplateFile.PageTemplateFile('manageMedianetEncoderProfile', _zptdir)
    
    security.declarePublic( 'testExpression' )
    def testExpression(self, object=None):

        """ Evaluate condition using context, 'ec', and return 0 or 1.
        """
        from Products.MyMediaManager.General import bandwidths
        self._expression = Expression('python:%s' % repr(bandwidths))
        return self.evaluateExpression(object=self)
    
    security.declarePrivate( 'evaluteExpression' )
    def evaluateExpression(self, expression, object=None):

        """ Evaluate condition using 'ec'.
        """
        if expression:
            zLOG.LOG(self.meta_type, zLOG.INFO, "evaluateExpression: %s" % str(expression))
            expression = Expression(expression)
            portal = getToolByName(self, 'portal_url').getPortalObject()
            if object is None or not hasattr(object, 'aq_base'):
                folder = portal
            else:
                folder = object
                # Search up the containment hierarchy until we find an
                # object that claims it's a folder.
                while folder is not None:
                    if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                        # found it.
                        break
                    else:
                        folder = aq_parent(aq_inner(folder))
            ec = createExprContext(folder, portal, object)
            #try:
            return expression(ec)
            #except:
            #    return 0
        else:
            return 1
    # profile management.
    # see MyMediaManager/interfaces/IEncoderPlugin.py for details.
        
    security.declarePublic('listServersByState')
    def listServersByState(self, state):
        """
        Returns a list containing server objects that can be used be a specific encoder
        """
        servers = []
        medianet = getToolByName(self, 'portal_medianet')
        for server in medianet.listServers(state):
            for provide in server._provides:
                if provide in self._provides and not server in servers:
                    servers.append(server)
        return servers

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setProfileStructure')
    def setProfileStructure(self, profilestructure):
        self._profilestructure = profilestructure
        self._p_changed = 1
    
    security.declarePublic('getProfileStructure')
    def getProfileStructure(self):
        """
        Returns the structure of the profile for this encoder type.
        """
        return self._profilestructure
    
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addProfileProperty')
    def manage_addProfileProperty(self, prop_id, prop_title, prop_type, REQUEST=None):
        """
        Adds a new property to the default profile.
        """
        struct = self.getProfileStructure()
        for p in struct:
            if p['id'] == prop_id:
                raise "ValueError", "The id '%s' is already in use"
        struct.append({'id':prop_id,'title':prop_title,'type':prop_type,'value':None})
        self.setProfileStructure(struct)

        if REQUEST is not None:
            msg = 'Added %s' % prop_id
            return self.manage_profiles(
                    self, REQUEST, manage_tabs_message=msg)
            
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_delProfileProperties')
    def manage_delProfileProperties(self, ids, REQUEST=None):
        """
        Deletes a property from the default profile.
        """
        if type(ids) not in (type(()), type([])):
            ids = (ids,)
        struct = self.getProfileStructure()
        new_struct = []
        deleted = []
        idx = 0
        #return ids
        for p in struct:
            if str(idx) not in ids:
                new_struct.append(p)
            else:
                deleted.append(p['id'])
            idx = idx + 1
            
        self.setProfileStructure(new_struct)
        
        if REQUEST is not None:
            msg = 'Deleted %s' % string.join(deleted, ',')
            return self.manage_profiles(
                    self, REQUEST, manage_tabs_message=msg)
    
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_editProfile')
    def manage_editProfile(self, REQUEST):
        """
        Saves the default profile structure.
        """
        #self.title = REQUEST.get('title','')
        profile = []
        
        valid = 1
        msg = ''
        
        for prop in self.getProfileStructure():
            p = REQUEST.get(prop['id'], '')
            prop['title'] = REQUEST.get('%s_title' % prop['id'], '')
            if prop['type'] == 'int':
                try:
                    prop['value'] = int(p)
                except: 
                    pass #raise "The value given to %s is not an integer" % prop['id']
            elif prop['type'] == 'boolean':
                if str(p).lower() in ('yes','no','true','false'):
                    if p in ('yes','true'):
                        prop['value'] = 1
                    else:
                        prop['value'] = 0
                try:
                    p = int(p)
                except: pass
                if p:
                    prop['value'] = 1
                else:
                    prop['value'] = 0
            elif prop['type'] in ('string','text'):
                prop['value'] = p
            elif prop['type'] == 'multiple selection':
                if type(p) not in (type(()), type([])):
                    p = (p,)
                expr = REQUEST['%s_expression' % prop['id']]
                expr = expr.strip()
                zLOG.LOG(self.meta_type, zLOG.INFO, "manage_editProfile. id: %s expression: %s" % (prop['id'], expr))
                if expr:
                    try:
                        Expression(expr)
                        prop['value'] = expr
                    except:
                        raise "Expression Error", "The expression '%s' is not valid. %s" % (expr, sys.exc_info()[1])
                prop['selected'] = list(p)
            elif prop['type'] == 'selection':
                prop['selected'] = p
                expr = REQUEST['%s_expression' % prop['id']]
                expr = expr.strip()
                zLOG.LOG(self.meta_type, zLOG.INFO, "manage_editProfile. id: %s expression: %s" % (prop['id'], expr))
                if expr:
                    try:
                        Expression(expr)
                        prop['value'] = expr
                    except:
                        raise "Expression Error", "The expression '%s' is not valid. %s" % (expr, sys.exc_info()[1])
            profile.append(prop)
            
        self.setProfileStructure(profile)
            
        msg = 'Saved'
        return self.manage_profiles(
                self, REQUEST, manage_tabs_message=msg)
    
