##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import zLOG, string
from utils import getUniqueId
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

class ServerInfo(SimpleItem):
    """
    A class that binds a server object to a masterfile
    """

    meta_type = 'Media Manager ServerInfo'
    security = ClassSecurityInfo()

    def __init__(self, masterfile, serverid):
        """
        Make a new instance of a ServerInfo object
        """
        self.id = getUniqueId()
        self._masterfile = masterfile
        self._content_type = None
        self._serverid = serverid
        self._encoder_profile = None
        self._cinemaskin = None
        self._syndicationskin = None
        self._encoder_start = None

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, REQUEST=None, datadict=None):
        """
        Edit the ServerInfo object
        """
        if REQUEST:
            req = REQUEST.form
            req_url = REQUEST.get('HTTP_REFERER', None)
            has_query = string.find(req_url, '?')
            if has_query != -1:
                req_url, query = string.split(req_url, '?')
        elif datadict:
            req = datadict
            req_url = None
        else:
            raise 'AttributeError', 'Missing data that should be used to update the serverinformation'

        for key in req.keys():
            if key == 'serverid':
                self._serverid = req[key]
            elif key == 'encoder_profile':
                self._encoder_profile = req[key]
            elif key == 'cinemaskin':
                self._cinemaskin = req[key]
            elif key == 'syndicationskin':
                self._syndicationskin = req[key]
            elif key == 'content_type':
                zLOG.LOG('Serverinfo - edit', zLOG.INFO, 'Content type is: %s' % req[key])
                self._content_type == req[key]
            elif key == 'edit':
                continue
            elif key == 'mediafile':
                filedata = req[key]
                if filedata:
                    self.uploadMasterfileData(filedata)
            elif key == 'filename':
                masterfile = self.getMasterfile()
                masterfile.edit(filename=req[key])
            else:
                setattr(self, '_%s' % key, req[key])

        msg = 'Server Information updated'
        if req_url:
            REQUEST.RESPONSE.redirect('%s?serverid=%s&portal_status_message=%s' % (req_url, self.getMediaServer().getId(), string.replace(msg, ' ', '+')))

    security.declarePublic('getValueFor')
    def getValueFor(self, attr):
        """
        Return the value for a specific attribute
        """
        value = getattr(self, '_%s' % attr, None)
        if not value:
            server = self.getMediaServer()
            value = getattr(server, '_%s' % attr, None)
        if value == 'None':
            value = None
        return value

    security.declarePublic('getContentType')
    def getContentType(self):
        """
        Return the content type of the content
        """
        if hasattr(self, '_content_type'):
            zLOG.LOG('Serverinfo', zLOG.INFO, 'has content type %s' % self._content_type)
            return self._content_type
        else:
            zLOG.LOG('Serverinfo', zLOG.INFO, 'no content type')
            return None

    security.declarePublic('getMasterfile')
    def getMasterfile(self):
        """
        Return the masterfile object for this serverinfo object
        """
        return self._masterfile

    security.declarePublic('uploadMasterfileData')
    def uploadMasterfileData(self, data):
        """
        Upload data to a masterfile
        """
        masterfile = self.getMasterfile()
        msg = masterfile.upload(data, self)
        
    security.declareProtected(CMFCorePermissions.ManageProperties, 'setEncoderStart')
    def setEncoderStart(self, encstart=None):
        """
        Set the encoder start time
        """
        if encstart is None:
            encstart = DateTime()
        self._encoder_start = encstart
        
    security.declarePublic('getEncoderStart')
    def getEncoderStart(self):
        """
        Return the encoder start time
        """
        return self._encoder_start

    security.declarePublic('getMediaServer')
    def getMediaServer(self):
        """
        Return the media server corresponding to the serverid
        """
        medianet = getToolByName(self, 'portal_medianet')
        return medianet.getServerById(self._serverid)

    security.declarePublic('getInfoAttrs')
    def getInfoAttrs(self):
        server = self.getMediaServer()
        serverattrs = server.getDataListForServerInfo()

        for attrdict in serverattrs:
            id = attrdict['id']
            infoattr = getattr(self, '_%s' % id, None)
            if infoattr:
                attrdict['value']= infoattr
        attrlist = serverattrs
        return attrlist
